@@ -1,7 +1,7 @@
 //! Contains the types needed to specify the auth configuration for a
 //! [`Client`](struct.Client.html).
 
-#[cfg(feature = "tokio-runtime")]
+#[cfg(feature = "aws-auth")]
 mod aws;
 mod plain;
 mod sasl;
@@ -82,7 +82,8 @@ pub enum AuthMechanism {
     ///
     /// Note: Only server versions 4.4+ support AWS authentication. Additionally, the driver only
     /// supports AWS authentication with the tokio runtime.
-    #[cfg(feature = "tokio-runtime")]
+    #[cfg(any(feature = "aws-auth", docsrs))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "aws-auth")))]
     MongoDbAws,
 }
 
@@ -114,7 +115,7 @@ impl AuthMechanism {
         match self {
             AuthMechanism::ScramSha1 | AuthMechanism::ScramSha256 => {
                 if credential.username.is_none() {
-                    return Err(ErrorKind::ArgumentError {
+                    return Err(ErrorKind::InvalidArgument {
                         message: "No username provided for SCRAM authentication".to_string(),
                     }
                     .into());
@@ -123,14 +124,14 @@ impl AuthMechanism {
             }
             AuthMechanism::MongoDbX509 => {
                 if credential.password.is_some() {
-                    return Err(ErrorKind::ArgumentError {
+                    return Err(ErrorKind::InvalidArgument {
                         message: "A password cannot be specified with MONGODB-X509".to_string(),
                     }
                     .into());
                 }
 
                 if credential.source.as_deref().unwrap_or("$external") != "$external" {
-                    return Err(ErrorKind::ArgumentError {
+                    return Err(ErrorKind::InvalidArgument {
                         message: "only $external may be specified as an auth source for \
                                   MONGODB-X509"
                             .to_string(),
@@ -142,21 +143,21 @@ impl AuthMechanism {
             }
             AuthMechanism::Plain => {
                 if credential.username.is_none() {
-                    return Err(ErrorKind::ArgumentError {
+                    return Err(ErrorKind::InvalidArgument {
                         message: "No username provided for PLAIN authentication".to_string(),
                     }
                     .into());
                 }
 
                 if credential.username.as_deref() == Some("") {
-                    return Err(ErrorKind::ArgumentError {
+                    return Err(ErrorKind::InvalidArgument {
                         message: "Username for PLAIN authentication must be non-empty".to_string(),
                     }
                     .into());
                 }
 
                 if credential.password.is_none() {
-                    return Err(ErrorKind::ArgumentError {
+                    return Err(ErrorKind::InvalidArgument {
                         message: "No password provided for PLAIN authentication".to_string(),
                     }
                     .into());
@@ -164,10 +165,10 @@ impl AuthMechanism {
 
                 Ok(())
             }
-            #[cfg(feature = "tokio-runtime")]
+            #[cfg(feature = "aws-auth")]
             AuthMechanism::MongoDbAws => {
                 if credential.username.is_some() && credential.password.is_none() {
-                    return Err(ErrorKind::ArgumentError {
+                    return Err(ErrorKind::InvalidArgument {
                         message: "Username cannot be provided without password for MONGODB-AWS \
                                   authentication"
                             .to_string(),
@@ -190,7 +191,7 @@ impl AuthMechanism {
             AuthMechanism::MongoDbX509 => MONGODB_X509_STR,
             AuthMechanism::Gssapi => GSSAPI_STR,
             AuthMechanism::Plain => PLAIN_STR,
-            #[cfg(feature = "tokio-runtime")]
+            #[cfg(feature = "aws-auth")]
             AuthMechanism::MongoDbAws => MONGODB_AWS_STR,
         }
     }
@@ -204,8 +205,8 @@ impl AuthMechanism {
                 uri_db.unwrap_or("admin")
             }
             AuthMechanism::MongoDbX509 => "$external",
-            AuthMechanism::Plain => "$external",
-            #[cfg(feature = "tokio-runtime")]
+            AuthMechanism::Plain => uri_db.unwrap_or("$external"),
+            #[cfg(feature = "aws-auth")]
             AuthMechanism::MongoDbAws => "$external",
             _ => "",
         }
@@ -229,19 +230,19 @@ impl AuthMechanism {
 
                 Ok(Some(ClientFirst::Scram(ScramVersion::Sha256, client_first)))
             }
-            Self::MongoDbX509 => Ok(Some(ClientFirst::X509(
+            Self::MongoDbX509 => Ok(Some(ClientFirst::X509(Box::new(
                 x509::build_speculative_client_first(credential),
-            ))),
+            )))),
             Self::Plain => Ok(None),
-            #[cfg(feature = "tokio-runtime")]
+            #[cfg(feature = "aws-auth")]
             AuthMechanism::MongoDbAws => Ok(None),
-            AuthMechanism::MongoDbCr => Err(ErrorKind::AuthenticationError {
+            AuthMechanism::MongoDbCr => Err(ErrorKind::Authentication {
                 message: "MONGODB-CR is deprecated and not supported by this driver. Use SCRAM \
                           for password-based authentication instead"
                     .into(),
             }
             .into()),
-            _ => Err(ErrorKind::AuthenticationError {
+            _ => Err(ErrorKind::Authentication {
                 message: format!("Authentication mechanism {:?} not yet implemented.", self),
             }
             .into()),
@@ -253,7 +254,7 @@ impl AuthMechanism {
         stream: &mut Connection,
         credential: &Credential,
         server_api: Option<&ServerApi>,
-        #[cfg_attr(not(feature = "tokio-runtime"), allow(unused))] http_client: &HttpClient,
+        #[cfg_attr(not(feature = "aws-auth"), allow(unused))] http_client: &HttpClient,
     ) -> Result<()> {
         self.validate_credential(credential)?;
 
@@ -274,17 +275,17 @@ impl AuthMechanism {
             AuthMechanism::Plain => {
                 plain::authenticate_stream(stream, credential, server_api).await
             }
-            #[cfg(feature = "tokio-runtime")]
+            #[cfg(feature = "aws-auth")]
             AuthMechanism::MongoDbAws => {
                 aws::authenticate_stream(stream, credential, server_api, http_client).await
             }
-            AuthMechanism::MongoDbCr => Err(ErrorKind::AuthenticationError {
+            AuthMechanism::MongoDbCr => Err(ErrorKind::Authentication {
                 message: "MONGODB-CR is deprecated and not supported by this driver. Use SCRAM \
                           for password-based authentication instead"
                     .into(),
             }
             .into()),
-            _ => Err(ErrorKind::AuthenticationError {
+            _ => Err(ErrorKind::Authentication {
                 message: format!("Authentication mechanism {:?} not yet implemented.", self),
             }
             .into()),
@@ -304,15 +305,17 @@ impl FromStr for AuthMechanism {
             GSSAPI_STR => Ok(AuthMechanism::Gssapi),
             PLAIN_STR => Ok(AuthMechanism::Plain),
 
-            #[cfg(feature = "tokio-runtime")]
+            #[cfg(feature = "aws-auth")]
             MONGODB_AWS_STR => Ok(AuthMechanism::MongoDbAws),
-            #[cfg(not(feature = "tokio-runtime"))]
-            MONGODB_AWS_STR => Err(ErrorKind::ArgumentError {
-                message: "MONGODB-AWS auth is only supported with the tokio runtime".into(),
+            #[cfg(not(feature = "aws-auth"))]
+            MONGODB_AWS_STR => Err(ErrorKind::InvalidArgument {
+                message: "MONGODB-AWS auth is only supported with the aws-auth feature flag and \
+                          the tokio runtime"
+                    .into(),
             }
             .into()),
 
-            _ => Err(ErrorKind::ArgumentError {
+            _ => Err(ErrorKind::InvalidArgument {
                 message: format!("invalid mechanism string: {}", str),
             }
             .into()),
@@ -325,29 +328,26 @@ impl FromStr for AuthMechanism {
 /// Some fields (mechanism and source) may be omitted and will either be negotiated or assigned a
 /// default value, depending on the values of other fields in the credential.
 #[derive(Clone, Default, Deserialize, TypedBuilder, PartialEq)]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct Credential {
     /// The username to authenticate with. This applies to all mechanisms but may be omitted when
     /// authenticating via MONGODB-X509.
-    #[builder(default)]
     pub username: Option<String>,
 
     /// The database used to authenticate. This applies to all mechanisms and defaults to "admin"
-    /// in SCRAM authentication mechanisms and "$external" for GSSAPI, MONGODB-X509 and PLAIN.
-    #[builder(default)]
+    /// in SCRAM authentication mechanisms, "$external" for GSSAPI and MONGODB-X509, and the
+    /// database name or "$external" for PLAIN.
     pub source: Option<String>,
 
     /// The password to authenticate with. This does not apply to all mechanisms.
-    #[builder(default)]
     pub password: Option<String>,
 
     /// Which authentication mechanism to use. If not provided, one will be negotiated with the
     /// server.
-    #[builder(default)]
     pub mechanism: Option<AuthMechanism>,
 
     /// Additional properties for the given mechanism.
-    #[builder(default)]
     pub mechanism_properties: Option<Document>,
 }
 
@@ -435,6 +435,31 @@ impl Credential {
             .authenticate_stream(conn, self, server_api, http_client)
             .await
     }
+
+    #[cfg(test)]
+    pub(crate) fn serialize_for_client_options<S>(
+        credential: &Option<Credential>,
+        serializer: S,
+    ) -> std::result::Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        use serde::ser::Serialize;
+
+        #[derive(serde::Serialize)]
+        struct CredentialHelper<'a> {
+            authsource: Option<&'a String>,
+            authmechanism: Option<&'a str>,
+            authmechanismproperties: Option<&'a Document>,
+        }
+
+        let state = credential.as_ref().map(|c| CredentialHelper {
+            authsource: c.source.as_ref(),
+            authmechanism: c.mechanism.as_ref().map(|s| s.as_str()),
+            authmechanismproperties: c.mechanism_properties.as_ref(),
+        });
+        state.serialize(serializer)
+    }
 }
 
 impl Debug for Credential {
@@ -448,7 +473,7 @@ impl Debug for Credential {
 /// Contains the first client message sent as part of the authentication handshake.
 pub(crate) enum ClientFirst {
     Scram(ScramVersion, scram::ClientFirst),
-    X509(Command),
+    X509(Box<Command>),
 }
 
 impl ClientFirst {
