@@ -1,5 +1,7 @@
 use derivative::Derivative;
 
+#[cfg(test)]
+use super::options::BackgroundThreadInterval;
 use super::{
     conn::PendingConnection,
     connection_requester,
@@ -11,7 +13,7 @@ use super::{
     },
     establish::ConnectionEstablisher,
     manager,
-    manager::{ManagementRequestReceiver, PoolManagementRequest, PoolManager},
+    manager::{ConnectionSucceeded, ManagementRequestReceiver, PoolManagementRequest, PoolManager},
     options::{ConnectionOptions, ConnectionPoolOptions},
     status,
     status::{PoolGenerationPublisher, PoolGenerationSubscriber},
@@ -19,7 +21,8 @@ use super::{
     DEFAULT_MAX_POOL_SIZE,
 };
 use crate::{
-    error::Result,
+    bson::oid::ObjectId,
+    error::{load_balanced_mode_mismatch, Error, ErrorKind, Result},
     event::cmap::{
         CmapEventHandler,
         ConnectionClosedEvent,
@@ -28,13 +31,17 @@ use crate::{
         PoolClosedEvent,
         PoolReadyEvent,
     },
-    options::StreamAddress,
+    options::ServerAddress,
     runtime::HttpClient,
     sdam::ServerUpdateSender,
     RUNTIME,
 };
 
-use std::{collections::VecDeque, sync::Arc, time::Duration};
+use std::{
+    collections::{HashMap, VecDeque},
+    sync::Arc,
+    time::Duration,
+};
 use tokio::sync::mpsc;
 
 const MAX_CONNECTING: u32 = 2;
@@ -45,7 +52,7 @@ const MAINTENACE_FREQUENCY: Duration = Duration::from_millis(500);
 #[derivative(Debug)]
 pub(crate) struct ConnectionPoolWorker {
     /// The address the pool's connections will connect to.
-    address: StreamAddress,
+    address: ServerAddress,
 
     /// Current state of the pool. Determines if connections may be checked out
     /// and if min_pool_size connection creation should continue.
@@ -64,7 +71,10 @@ pub(crate) struct ConnectionPoolWorker {
     /// The current generation of the pool. The generation is incremented whenever the pool is
     /// cleared. Connections belonging to a previous generation are considered stale and will be
     /// closed when checked back in or when popped off of the set of available connections.
-    generation: u32,
+    generation: PoolGeneration,
+
+    /// The connection count for each serviceId in load-balanced mode.
+    service_connection_count: HashMap<ObjectId, u32>,
 
     /// The established connections that are currently checked into the pool and awaiting usage in
     /// future operations.
@@ -132,13 +142,15 @@ impl ConnectionPoolWorker {
     /// Once all connection requesters are dropped, the worker will stop executing
     /// and close the pool.
     pub(super) fn start(
-        address: StreamAddress,
+        address: ServerAddress,
         http_client: HttpClient,
         server_updater: ServerUpdateSender,
         options: Option<ConnectionPoolOptions>,
     ) -> (PoolManager, ConnectionRequester, PoolGenerationSubscriber) {
         let establisher = ConnectionEstablisher::new(http_client, options.as_ref());
-        let event_handler = options.as_ref().and_then(|opts| opts.event_handler.clone());
+        let event_handler = options
+            .as_ref()
+            .and_then(|opts| opts.cmap_event_handler.clone());
 
         // The CMAP spec indicates that a max idle time of zero means that connections should not be
         // closed due to idleness.
@@ -164,24 +176,46 @@ impl ConnectionPoolWorker {
         let (manager, management_receiver) = manager::channel();
         let (generation_publisher, generation_subscriber) = status::channel();
 
+        let is_load_balanced = options
+            .as_ref()
+            .and_then(|opts| opts.load_balanced)
+            .unwrap_or(false);
+        let generation = if is_load_balanced {
+            PoolGeneration::load_balanced()
+        } else {
+            PoolGeneration::normal()
+        };
+
         #[cfg(test)]
-        let state = if options
+        let mut state = if options
             .as_ref()
             .and_then(|opts| opts.ready)
             .unwrap_or(false)
         {
             PoolState::Ready
         } else {
-            PoolState::Paused
+            PoolState::New
         };
         #[cfg(test)]
         let maintenance_frequency = options
             .as_ref()
-            .and_then(|opts| opts.maintenance_frequency)
+            .and_then(|opts| opts.background_thread_interval)
+            .map(|i| match i {
+                // One year is long enough to count as never for tests, but not so long that it
+                // will overflow interval math.
+                BackgroundThreadInterval::Never => Duration::from_secs(31_556_952),
+                BackgroundThreadInterval::Every(d) => d,
+            })
             .unwrap_or(MAINTENACE_FREQUENCY);
 
         #[cfg(not(test))]
-        let (state, maintenance_frequency) = (PoolState::Paused, MAINTENACE_FREQUENCY);
+        let (mut state, maintenance_frequency) = (PoolState::New, MAINTENACE_FREQUENCY);
+
+        if is_load_balanced {
+            // Because load balancer servers don't have a monitoring connection, the associated
+            // connection pool needs start in the ready state.
+            state = PoolState::Ready;
+        }
 
         let worker = ConnectionPoolWorker {
             address,
@@ -192,7 +226,8 @@ impl ConnectionPoolWorker {
             next_connection_id: 1,
             total_connection_count: 0,
             pending_connection_count: 0,
-            generation: 0,
+            generation,
+            service_connection_count: HashMap::new(),
             connection_options,
             available_connections: VecDeque::new(),
             max_pool_size,
@@ -244,27 +279,43 @@ impl ConnectionPoolWorker {
                     PoolState::Ready => {
                         self.wait_queue.push_back(request);
                     }
-                    PoolState::Paused => {
+                    PoolState::Paused(ref e) => {
                         // if receiver doesn't listen to error that's ok.
-                        let _ = request.fulfill(ConnectionRequestResult::PoolCleared);
+                        let _ = request.fulfill(ConnectionRequestResult::PoolCleared(e.clone()));
+                    }
+                    PoolState::New => {
+                        let _ = request.fulfill(ConnectionRequestResult::PoolCleared(
+                            ErrorKind::Internal {
+                                message: "check out attempted from new pool".to_string(),
+                            }
+                            .into(),
+                        ));
                     }
                 },
                 PoolTask::HandleManagementRequest(PoolManagementRequest::CheckIn(connection)) => {
-                    self.check_in(connection)
+                    self.check_in(connection);
+                }
+                PoolTask::HandleManagementRequest(PoolManagementRequest::Clear {
+                    completion_handler: _,
+                    cause,
+                    service_id,
+                }) => {
+                    self.clear(cause, service_id);
                 }
-                PoolTask::HandleManagementRequest(PoolManagementRequest::Clear) => self.clear(),
                 PoolTask::HandleManagementRequest(PoolManagementRequest::MarkAsReady {
                     completion_handler: _handler,
                 }) => {
                     self.mark_as_ready();
                 }
                 PoolTask::HandleManagementRequest(
-                    PoolManagementRequest::HandleConnectionSucceeded(c),
-                ) => self.handle_connection_succeeded(c),
+                    PoolManagementRequest::HandleConnectionSucceeded(conn),
+                ) => self.handle_connection_succeeded(conn),
                 PoolTask::HandleManagementRequest(
                     PoolManagementRequest::HandleConnectionFailed,
                 ) => self.handle_connection_failed(),
-                PoolTask::Maintenance => self.perform_maintenance(),
+                PoolTask::Maintenance => {
+                    self.perform_maintenance();
+                }
             }
 
             if self.can_service_connection_request() {
@@ -302,7 +353,7 @@ impl ConnectionPoolWorker {
         // first attempt to check out an available connection
         while let Some(mut conn) = self.available_connections.pop_back() {
             // Close the connection if it's stale.
-            if conn.is_stale(self.generation) {
+            if conn.generation.is_stale(&self.generation) {
                 self.close_connection(conn, ConnectionClosedReason::Stale);
                 continue;
             }
@@ -345,7 +396,9 @@ impl ConnectionPoolWorker {
 
                 if let Ok(ref mut c) = establish_result {
                     c.mark_as_in_use(manager.clone());
-                    manager.handle_connection_succeeded(None);
+                    manager.handle_connection_succeeded(ConnectionSucceeded::Used {
+                        service_id: c.generation.service_id(),
+                    });
                 }
 
                 establish_result
@@ -379,7 +432,7 @@ impl ConnectionPoolWorker {
         let pending_connection = PendingConnection {
             id: self.next_connection_id,
             address: self.address.clone(),
-            generation: self.generation,
+            generation: self.generation.clone(),
             options: self.connection_options.clone(),
         };
         self.next_connection_id += 1;
@@ -400,9 +453,13 @@ impl ConnectionPoolWorker {
 
     /// Process a successful connection establishment, optionally populating the pool with the
     /// resulting connection.
-    fn handle_connection_succeeded(&mut self, connection: Option<Connection>) {
+    fn handle_connection_succeeded(&mut self, connection: ConnectionSucceeded) {
         self.pending_connection_count -= 1;
-        if let Some(mut connection) = connection {
+        if let Some(sid) = connection.service_id() {
+            let count = self.service_connection_count.entry(sid).or_insert(0);
+            *count += 1;
+        }
+        if let ConnectionSucceeded::ForPool(mut connection) = connection {
             connection.mark_as_available();
             self.available_connections.push_back(connection);
         }
@@ -417,21 +474,32 @@ impl ConnectionPoolWorker {
 
         if conn.has_errored() {
             self.close_connection(conn, ConnectionClosedReason::Error);
-        } else if conn.is_stale(self.generation) {
+        } else if conn.generation.is_stale(&self.generation) {
             self.close_connection(conn, ConnectionClosedReason::Stale);
         } else if conn.is_executing() {
-            self.close_connection(conn, ConnectionClosedReason::Dropped)
+            self.close_connection(conn, ConnectionClosedReason::Dropped);
         } else {
             self.available_connections.push_back(conn);
         }
     }
 
-    fn clear(&mut self) {
-        self.generation += 1;
-        let previous_state = std::mem::replace(&mut self.state, PoolState::Paused);
-        self.generation_publisher.publish(self.generation);
+    fn clear(&mut self, cause: Error, service_id: Option<ObjectId>) {
+        let was_ready = match (&mut self.generation, service_id) {
+            (PoolGeneration::Normal(gen), None) => {
+                *gen += 1;
+                let prev = std::mem::replace(&mut self.state, PoolState::Paused(cause.clone()));
+                matches!(prev, PoolState::Ready)
+            }
+            (PoolGeneration::LoadBalanced(gen_map), Some(sid)) => {
+                let gen = gen_map.entry(sid).or_insert(0);
+                *gen += 1;
+                true
+            }
+            (..) => load_balanced_mode_mismatch!(),
+        };
+        self.generation_publisher.publish(self.generation.clone());
 
-        if !matches!(previous_state, PoolState::Paused) {
+        if was_ready {
             self.emit_event(|handler| {
                 let event = PoolClearedEvent {
                     address: self.address.clone(),
@@ -440,11 +508,13 @@ impl ConnectionPoolWorker {
                 handler.handle_pool_cleared_event(event);
             });
 
-            for request in self.wait_queue.drain(..) {
-                // an error means the other end hung up already, which is okay because we were
-                // returning an error anyways
-                let _: std::result::Result<_, _> =
-                    request.fulfill(ConnectionRequestResult::PoolCleared);
+            if !self.generation.is_load_balanced() {
+                for request in self.wait_queue.drain(..) {
+                    // an error means the other end hung up already, which is okay because we were
+                    // returning an error anyways
+                    let _: std::result::Result<_, _> =
+                        request.fulfill(ConnectionRequestResult::PoolCleared(cause.clone()));
+                }
             }
         }
     }
@@ -475,7 +545,24 @@ impl ConnectionPoolWorker {
 
     /// Close a connection, emit the event for it being closed, and decrement the
     /// total connection count.
+    #[allow(clippy::single_match)]
     fn close_connection(&mut self, connection: Connection, reason: ConnectionClosedReason) {
+        match (&mut self.generation, connection.generation.service_id()) {
+            (PoolGeneration::LoadBalanced(gen_map), Some(sid)) => {
+                match self.service_connection_count.get_mut(&sid) {
+                    Some(count) => {
+                        *count -= 1;
+                        if *count == 0 {
+                            gen_map.remove(&sid);
+                            self.service_connection_count.remove(&sid);
+                        }
+                    }
+                    None => load_balanced_mode_mismatch!(),
+                }
+            }
+            (PoolGeneration::Normal(_), None) => {}
+            _ => load_balanced_mode_mismatch!(),
+        }
         connection.close_and_drop(reason);
         self.total_connection_count -= 1;
     }
@@ -492,7 +579,7 @@ impl ConnectionPoolWorker {
     /// Iterate over the connections and remove any that are stale or idle.
     fn remove_perished_connections(&mut self) {
         while let Some(connection) = self.available_connections.pop_front() {
-            if connection.is_stale(self.generation) {
+            if connection.generation.is_stale(&self.generation) {
                 // the following unwrap is okay becaue we asserted the pool was nonempty
                 self.close_connection(connection, ConnectionClosedReason::Stale);
             } else if connection.is_idle(self.max_idle_time) {
@@ -528,7 +615,8 @@ impl ConnectionPoolWorker {
                     .await;
 
                     if let Ok(connection) = connection {
-                        manager.handle_connection_succeeded(Some(connection))
+                        manager
+                            .handle_connection_succeeded(ConnectionSucceeded::ForPool(connection))
                     }
                 });
             }
@@ -547,7 +635,6 @@ async fn establish_connection(
     event_handler: Option<&Arc<dyn CmapEventHandler>>,
 ) -> Result<Connection> {
     let connection_id = pending_connection.id;
-    let generation = pending_connection.generation;
     let address = pending_connection.address.clone();
 
     let mut establish_result = establisher.establish_connection(pending_connection).await;
@@ -562,7 +649,7 @@ async fn establish_connection(
                 };
                 handler.handle_connection_closed_event(event);
             }
-            server_updater.handle_error(e.clone(), generation).await;
+            server_updater.handle_error(e.clone()).await;
             manager.handle_connection_failed();
         }
         Ok(ref mut connection) => {
@@ -572,7 +659,7 @@ async fn establish_connection(
         }
     }
 
-    establish_result
+    establish_result.map_err(|e| e.cause)
 }
 
 /// Enum modeling the possible pool states as described in the CMAP spec.
@@ -581,8 +668,11 @@ async fn establish_connection(
 /// once it goes out of scope and cannot be manually closed before then.
 #[derive(Debug)]
 enum PoolState {
+    /// Same as Paused, but only for a new pool, not one that has been cleared due to an error.
+    New,
+
     /// Connections may not be checked out nor created in the background to satisfy minPoolSize.
-    Paused,
+    Paused(Error),
 
     /// Pool is operational.
     Ready,
@@ -643,3 +733,31 @@ impl HandleListener {
         self.receiver.recv().await;
     }
 }
+
+#[derive(Debug, Clone)]
+pub(crate) enum PoolGeneration {
+    Normal(u32),
+    LoadBalanced(HashMap<ObjectId, u32>),
+}
+
+impl PoolGeneration {
+    pub(crate) fn normal() -> Self {
+        Self::Normal(0)
+    }
+
+    fn load_balanced() -> Self {
+        Self::LoadBalanced(HashMap::new())
+    }
+
+    fn is_load_balanced(&self) -> bool {
+        matches!(self, Self::LoadBalanced(_))
+    }
+
+    #[cfg(test)]
+    pub(crate) fn as_normal(&self) -> Option<u32> {
+        match self {
+            PoolGeneration::Normal(n) => Some(*n),
+            _ => None,
+        }
+    }
+}
