@@ -1,17 +1,13 @@
 use std::time::Duration;
 
+use bson::serde_helpers;
 use serde::{de::Error, Deserialize, Deserializer, Serialize, Serializer};
 use serde_with::skip_serializing_none;
 use typed_builder::TypedBuilder;
 
 use crate::{
     bson::{doc, Bson, Document},
-    bson_util::{
-        deserialize_duration_from_u64_millis,
-        serialize_batch_size,
-        serialize_duration_as_int_millis,
-        serialize_u32_as_i32,
-    },
+    bson_util,
     concern::{ReadConcern, WriteConcern},
     options::Collation,
     selection_criteria::SelectionCriteria,
@@ -20,19 +16,17 @@ use crate::{
 /// These are the valid options for creating a [`Collection`](../struct.Collection.html) with
 /// [`Database::collection_with_options`](../struct.Database.html#method.collection_with_options).
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
+#[builder(field_defaults(default, setter(into)))]
 #[serde(rename_all = "camelCase")]
 #[non_exhaustive]
 pub struct CollectionOptions {
     /// The default read preference for operations.
-    #[builder(default)]
     pub selection_criteria: Option<SelectionCriteria>,
 
     /// The default read concern for operations.
-    #[builder(default)]
     pub read_concern: Option<ReadConcern>,
 
     /// The default write concern for operations.
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 }
 
@@ -103,14 +97,13 @@ pub enum CursorType {
 /// [`Collection::insert_one`](../struct.Collection.html#method.insert_one) operation.
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct InsertOneOptions {
     /// Opt out of document-level validation.
-    #[builder(default)]
     pub bypass_document_validation: Option<bool>,
 
     /// The write concern for the operation.
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 }
 
@@ -118,22 +111,20 @@ pub struct InsertOneOptions {
 /// [`Collection::insert_many`](../struct.Collection.html#method.insert_many) operation.
 #[skip_serializing_none]
 #[derive(Clone, Debug, Default, TypedBuilder, Serialize, Deserialize)]
+#[builder(field_defaults(default, setter(into)))]
 #[serde(rename_all = "camelCase")]
 #[non_exhaustive]
 pub struct InsertManyOptions {
     /// Opt out of document-level validation.
-    #[builder(default)]
     pub bypass_document_validation: Option<bool>,
 
     /// If true, when an insert fails, return without performing the remaining writes. If false,
     /// when a write fails, continue with the remaining writes, if any.
     ///
     /// Defaults to true.
-    #[builder(default)]
     pub ordered: Option<bool>,
 
     /// The write concern for the operation.
-    #[builder(default)]
     #[serde(skip_deserializing)]
     pub write_concern: Option<WriteConcern>,
 }
@@ -191,39 +182,34 @@ impl From<Vec<Document>> for UpdateModifications {
 /// [`Collection::update_many`](../struct.Collection.html#method.update_many) operation.
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct UpdateOptions {
     /// A set of filters specifying to which array elements an update should apply.
     ///
     /// See the documentation [here](https://docs.mongodb.com/manual/reference/command/update/) for
     /// more information on array filters.
-    #[builder(default)]
     pub array_filters: Option<Vec<Document>>,
 
     /// Opt out of document-level validation.
-    #[builder(default)]
     pub bypass_document_validation: Option<bool>,
 
     /// If true, insert a document if no matching document is found.
-    #[builder(default)]
     pub upsert: Option<bool>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// A document or string that specifies the index to use to support the query predicate.
     ///
     /// Only available in MongoDB 4.2+. See the official MongoDB
     /// [documentation](https://docs.mongodb.com/manual/reference/command/update/#ex-update-command-hint) for examples.
-    #[builder(default)]
     pub hint: Option<Hint>,
 
     /// The write concern for the operation.
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 }
 
@@ -244,32 +230,28 @@ impl UpdateOptions {
 /// [`Collection::replace_one`](../struct.Collection.html#method.replace_one) operation.
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct ReplaceOptions {
     /// Opt out of document-level validation.
-    #[builder(default)]
     pub bypass_document_validation: Option<bool>,
 
     /// If true, insert a document if no matching document is found.
-    #[builder(default)]
     pub upsert: Option<bool>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// A document or string that specifies the index to use to support the query predicate.
     ///
     /// Only available in MongoDB 4.2+. See the official MongoDB
     /// [documentation](https://docs.mongodb.com/manual/reference/command/update/#ex-update-command-hint) for examples.
-    #[builder(default)]
     pub hint: Option<Hint>,
 
     /// The write concern for the operation.
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 }
 
@@ -279,22 +261,20 @@ pub struct ReplaceOptions {
 #[serde_with::skip_serializing_none]
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder, Serialize)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct DeleteOptions {
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// The write concern for the operation.
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 
     /// The index to use for the operation.
     /// Only available in MongoDB 4.4+.
-    #[builder(default)]
     pub hint: Option<Hint>,
 }
 
@@ -303,37 +283,32 @@ pub struct DeleteOptions {
 /// operation.
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct FindOneAndDeleteOptions {
     /// The maximum amount of time to allow the query to run.
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
     pub max_time: Option<Duration>,
 
     /// Limits the fields of the document being returned.
-    #[builder(default)]
     pub projection: Option<Document>,
 
     /// The order of the documents for the purposes of the operation.
-    #[builder(default)]
     pub sort: Option<Document>,
 
     /// The level of the write concern
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// The index to use for the operation.
     /// Only available in MongoDB 4.4+.
-    #[builder(default)]
     pub hint: Option<Hint>,
 }
 
@@ -341,50 +316,42 @@ pub struct FindOneAndDeleteOptions {
 /// [`Collection::find_one_and_replace`](../struct.Collection.html#method.find_one_and_replace)
 /// operation.
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
+#[builder(field_defaults(default, setter(into)))]
 #[serde(rename_all = "camelCase")]
 #[non_exhaustive]
 pub struct FindOneAndReplaceOptions {
     /// Opt out of document-level validation.
-    #[builder(default)]
     pub bypass_document_validation: Option<bool>,
 
     /// The maximum amount of time to allow the query to run.
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
     pub max_time: Option<Duration>,
 
     /// Limits the fields of the document being returned.
-    #[builder(default)]
     pub projection: Option<Document>,
 
     /// Whether the operation should return the document before or after modification.
-    #[builder(default)]
     pub return_document: Option<ReturnDocument>,
 
     /// The order of the documents for the purposes of the operation.
-    #[builder(default)]
     pub sort: Option<Document>,
 
     /// If true, insert a document if no matching document is found.
-    #[builder(default)]
     pub upsert: Option<bool>,
 
     /// The level of the write concern
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// The index to use for the operation.
     /// Only available in MongoDB 4.4+.
-    #[builder(default)]
     pub hint: Option<Hint>,
 }
 
@@ -393,56 +360,47 @@ pub struct FindOneAndReplaceOptions {
 /// operation.
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct FindOneAndUpdateOptions {
     /// A set of filters specifying to which array elements an update should apply.
     ///
     /// See the documentation [here](https://docs.mongodb.com/manual/reference/command/update/) for
     /// more information on array filters.
-    #[builder(default)]
     pub array_filters: Option<Vec<Document>>,
 
     /// Opt out of document-level validation.
-    #[builder(default)]
     pub bypass_document_validation: Option<bool>,
 
     /// The maximum amount of time to allow the query to run.
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
     pub max_time: Option<Duration>,
 
     /// Limits the fields of the document being returned.
-    #[builder(default)]
     pub projection: Option<Document>,
 
     /// Whether the operation should return the document before or after modification.
-    #[builder(default)]
     pub return_document: Option<ReturnDocument>,
 
     /// The order of the documents for the purposes of the operation.
-    #[builder(default)]
     pub sort: Option<Document>,
 
     /// If true, insert a document if no matching document is found.
-    #[builder(default)]
     pub upsert: Option<bool>,
 
     /// The level of the write concern
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// The index to use for the operation.
     /// Only available in MongoDB 4.4+.
-    #[builder(default)]
     pub hint: Option<Hint>,
 }
 
@@ -451,11 +409,11 @@ pub struct FindOneAndUpdateOptions {
 #[skip_serializing_none]
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder, Serialize)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct AggregateOptions {
     /// Enables writing to temporary files. When set to true, aggregation stages can write data to
     /// the _tmp subdirectory in the dbPath directory.
-    #[builder(default)]
     pub allow_disk_use: Option<bool>,
 
     /// The number of documents the server should return per cursor batch.
@@ -464,38 +422,36 @@ pub struct AggregateOptions {
     /// only the number of documents kept in memory at a given time (and by extension, the
     /// number of round trips needed to return the entire set of documents returned by the
     /// query).
-    #[builder(default)]
-    #[serde(serialize_with = "serialize_batch_size", rename(serialize = "cursor"))]
+    #[serde(
+        serialize_with = "bson_util::serialize_u32_option_as_batch_size",
+        rename(serialize = "cursor")
+    )]
     pub batch_size: Option<u32>,
 
     /// Opt out of document-level validation.
-    #[builder(default)]
     pub bypass_document_validation: Option<bool>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// Tags the query with an arbitrary string to help trace the operation through the database
     /// profiler, currentOp and logs.
-    #[builder(default)]
     pub comment: Option<String>,
 
     /// The index to use for the operation.
-    #[builder(default)]
     pub hint: Option<Hint>,
 
     /// The maximum amount of time for the server to wait on new documents to satisfy a tailable
     /// await cursor query.
     ///
     /// This option will have no effect on non-tailable cursors that result from this operation.
-    #[builder(default)]
     #[serde(
         skip_serializing,
-        deserialize_with = "deserialize_duration_from_u64_millis"
+        deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis",
+        default
     )]
     pub max_await_time: Option<Duration>,
 
@@ -503,11 +459,11 @@ pub struct AggregateOptions {
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
     #[serde(
-        serialize_with = "serialize_duration_as_int_millis",
+        serialize_with = "bson_util::serialize_duration_option_as_int_millis",
         rename = "maxTimeMS",
-        deserialize_with = "deserialize_duration_from_u64_millis"
+        deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis",
+        default
     )]
     pub max_time: Option<Duration>,
 
@@ -516,13 +472,13 @@ pub struct AggregateOptions {
     /// If none is specified, the read concern defined on the object executing this operation will
     /// be used.
     #[builder(default)]
+    #[serde(default)]
     pub read_concern: Option<ReadConcern>,
 
     /// The criteria used to select a server for this operation.
     ///
     /// If none is specified, the selection criteria defined on the object executing this operation
     /// will be used.
-    #[builder(default)]
     #[serde(skip_serializing)]
     #[serde(rename = "readPreference")]
     pub selection_criteria: Option<SelectionCriteria>,
@@ -531,52 +487,56 @@ pub struct AggregateOptions {
     ///
     /// If none is specified, the write concern defined on the object executing this operation will
     /// be used.
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
+
+    /// A document with any amount of parameter names, each followed by definitions of constants in
+    /// the MQL Aggregate Expression language.  Each parameter name is then usable to access the
+    /// value of the corresponding MQL Expression with the "$$" syntax within Aggregate Expression
+    /// contexts.
+    ///
+    /// This feature is only available on server versions 5.0 and above.
+    #[serde(rename = "let")]
+    pub let_vars: Option<Document>,
 }
 
 /// Specifies the options to a
 /// [`Collection::count_documents`](../struct.Collection.html#method.count_documents) operation.
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct CountOptions {
     /// The index to use for the operation.
-    #[builder(default)]
     pub hint: Option<Hint>,
 
     /// The maximum number of documents to count.
-    #[builder(default)]
-    pub limit: Option<i64>,
+    pub limit: Option<u64>,
 
     /// The maximum amount of time to allow the query to run.
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
-    #[serde(deserialize_with = "deserialize_duration_from_u64_millis")]
+    #[serde(
+        default,
+        deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis"
+    )]
     pub max_time: Option<Duration>,
 
     /// The number of documents to skip before counting.
-    #[builder(default)]
-    pub skip: Option<i64>,
+    pub skip: Option<u64>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// The criteria used to select a server for this operation.
     ///
     /// If none specified, the default set on the collection will be used.
-    #[builder(default)]
-    #[serde(skip_serializing)]
     pub selection_criteria: Option<SelectionCriteria>,
 
     /// The level of the read concern.
-    #[builder(default)]
     pub read_concern: Option<ReadConcern>,
 }
 
@@ -590,29 +550,28 @@ pub struct CountOptions {
 #[serde_with::skip_serializing_none]
 #[derive(Debug, Default, Deserialize, TypedBuilder, Serialize, Clone)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct EstimatedDocumentCountOptions {
     /// The maximum amount of time to allow the query to run.
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
     #[serde(
-        serialize_with = "serialize_duration_as_int_millis",
+        default,
+        serialize_with = "bson_util::serialize_duration_option_as_int_millis",
         rename = "maxTimeMS",
-        deserialize_with = "deserialize_duration_from_u64_millis"
+        deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis"
     )]
     pub max_time: Option<Duration>,
 
     /// The criteria used to select a server for this operation.
     ///
     /// If none specified, the default set on the collection will be used.
-    #[builder(default)]
     #[serde(skip_serializing)]
     pub selection_criteria: Option<SelectionCriteria>,
 
     /// The level of the read concern.
-    #[builder(default)]
     pub read_concern: Option<ReadConcern>,
 }
 
@@ -620,6 +579,7 @@ pub struct EstimatedDocumentCountOptions {
 /// operation.
 #[serde_with::skip_serializing_none]
 #[derive(Debug, Default, Deserialize, TypedBuilder, Serialize, Clone)]
+#[builder(field_defaults(default, setter(into)))]
 #[serde(rename_all = "camelCase")]
 #[non_exhaustive]
 pub struct DistinctOptions {
@@ -627,30 +587,27 @@ pub struct DistinctOptions {
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
     #[serde(
-        serialize_with = "serialize_duration_as_int_millis",
+        default,
+        serialize_with = "bson_util::serialize_duration_option_as_int_millis",
         rename = "maxTimeMS",
-        deserialize_with = "deserialize_duration_from_u64_millis"
+        deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis"
     )]
     pub max_time: Option<Duration>,
 
     /// The criteria used to select a server for this operation.
     ///
     /// If none specified, the default set on the collection will be used.
-    #[builder(default)]
     #[serde(skip_serializing)]
     pub selection_criteria: Option<SelectionCriteria>,
 
     /// The level of the read concern.
-    #[builder(default)]
     pub read_concern: Option<ReadConcern>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 }
 
@@ -658,18 +615,17 @@ pub struct DistinctOptions {
 /// operation.
 #[skip_serializing_none]
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder, Serialize)]
+#[builder(field_defaults(default, setter(into)))]
 #[serde(rename_all = "camelCase")]
 #[non_exhaustive]
 pub struct FindOptions {
     /// Enables writing to temporary files by the server. When set to true, the find operation can
     /// write data to the _tmp subdirectory in the dbPath directory. Only supported in server
     /// versions 4.4+.
-    #[builder(default)]
     pub allow_disk_use: Option<bool>,
 
     /// If true, partial results will be returned from a mongos rather than an error being
     /// returned if one or more shards is down.
-    #[builder(default)]
     pub allow_partial_results: Option<bool>,
 
     /// The number of documents the server should return per cursor batch.
@@ -678,38 +634,31 @@ pub struct FindOptions {
     /// only the number of documents kept in memory at a given time (and by extension, the
     /// number of round trips needed to return the entire set of documents returned by the
     /// query.
-    #[builder(default)]
-    #[serde(serialize_with = "serialize_u32_as_i32")]
+    #[serde(serialize_with = "bson_util::serialize_u32_option_as_i32")]
     pub batch_size: Option<u32>,
 
     /// Tags the query with an arbitrary string to help trace the operation through the database
     /// profiler, currentOp and logs.
-    #[builder(default)]
     pub comment: Option<String>,
 
     /// The type of cursor to return.
-    #[builder(default)]
     #[serde(skip)]
     pub cursor_type: Option<CursorType>,
 
     /// The index to use for the operation.
-    #[builder(default)]
     pub hint: Option<Hint>,
 
     /// The maximum number of documents to query.
     /// If a negative number is specified, the documents will be returned in a single batch limited
     /// in number by the positive value of the specified limit.
-    #[builder(default)]
     #[serde(serialize_with = "serialize_absolute_value")]
     pub limit: Option<i64>,
 
     /// The exclusive upper bound for a specific index.
-    #[builder(default)]
     pub max: Option<Document>,
 
     /// The maximum amount of time for the server to wait on new documents to satisfy a tailable
     /// cursor query. If the cursor is not tailable, this option is ignored.
-    #[builder(default)]
     #[serde(skip)]
     pub max_await_time: Option<Duration>,
 
@@ -717,66 +666,56 @@ pub struct FindOptions {
     ///
     /// Note: this option is deprecated starting in MongoDB version 4.0 and removed in MongoDB 4.2.
     /// Use the maxTimeMS option instead.
-    #[builder(default)]
-    pub max_scan: Option<i64>,
+    #[serde(serialize_with = "bson_util::serialize_u64_option_as_i64")]
+    pub max_scan: Option<u64>,
 
     /// The maximum amount of time to allow the query to run.
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
     #[serde(
         rename = "maxTimeMS",
-        serialize_with = "serialize_duration_as_int_millis"
+        serialize_with = "bson_util::serialize_duration_option_as_int_millis"
     )]
     pub max_time: Option<Duration>,
 
     /// The inclusive lower bound for a specific index.
-    #[builder(default)]
     pub min: Option<Document>,
 
     /// Whether the server should close the cursor after a period of inactivity.
-    #[builder(default)]
     pub no_cursor_timeout: Option<bool>,
 
     /// Limits the fields of the document being returned.
-    #[builder(default)]
     pub projection: Option<Document>,
 
     /// The read concern to use for this find query.
     ///
     /// If none specified, the default set on the collection will be used.
-    #[builder(default)]
     pub read_concern: Option<ReadConcern>,
 
     /// Whether to return only the index keys in the documents.
-    #[builder(default)]
     pub return_key: Option<bool>,
 
     /// The criteria used to select a server for this find query.
     ///
     /// If none specified, the default set on the collection will be used.
-    #[builder(default)]
     #[serde(skip)]
     pub selection_criteria: Option<SelectionCriteria>,
 
     /// Whether to return the record identifier for each document.
-    #[builder(default)]
     pub show_record_id: Option<bool>,
 
     /// The number of documents to skip before counting.
-    #[builder(default)]
-    pub skip: Option<i64>,
+    #[serde(serialize_with = "bson_util::serialize_u64_option_as_i64")]
+    pub skip: Option<u64>,
 
     /// The order of the documents for the purposes of the operation.
-    #[builder(default)]
     pub sort: Option<Document>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 }
 
@@ -800,7 +739,7 @@ impl From<FindOneOptions> for FindOptions {
             skip: options.skip,
             batch_size: None,
             cursor_type: None,
-            limit: None,
+            limit: Some(-1),
             max_await_time: None,
             no_cursor_timeout: None,
             sort: options.sort,
@@ -826,92 +765,222 @@ where
 /// operation.
 #[derive(Clone, Debug, Default, Deserialize, TypedBuilder)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct FindOneOptions {
     /// If true, partial results will be returned from a mongos rather than an error being
     /// returned if one or more shards is down.
-    #[builder(default)]
     pub allow_partial_results: Option<bool>,
 
     /// The collation to use for the operation.
     ///
     /// See the [documentation](https://docs.mongodb.com/manual/reference/collation/) for more
     /// information on how to use this option.
-    #[builder(default)]
     pub collation: Option<Collation>,
 
     /// Tags the query with an arbitrary string to help trace the operation through the database
     /// profiler, currentOp and logs.
-    #[builder(default)]
     pub comment: Option<String>,
 
     /// The index to use for the operation.
-    #[builder(default)]
     pub hint: Option<Hint>,
 
     /// The exclusive upper bound for a specific index.
-    #[builder(default)]
     pub max: Option<Document>,
 
     /// Maximum number of documents or index keys to scan when executing the query.
     ///
     /// Note: this option is deprecated starting in MongoDB version 4.0 and removed in MongoDB 4.2.
     /// Use the maxTimeMS option instead.
-    #[builder(default)]
-    pub max_scan: Option<i64>,
+    #[serde(serialize_with = "bson_util::serialize_u64_option_as_i64")]
+    pub max_scan: Option<u64>,
 
     /// The maximum amount of time to allow the query to run.
     ///
     /// This options maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
     /// across the wire as an integer number of milliseconds.
-    #[builder(default)]
-    #[serde(deserialize_with = "deserialize_duration_from_u64_millis")]
+    #[serde(deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis")]
     pub max_time: Option<Duration>,
 
     /// The inclusive lower bound for a specific index.
-    #[builder(default)]
     pub min: Option<Document>,
 
     /// Limits the fields of the document being returned.
-    #[builder(default)]
     pub projection: Option<Document>,
 
     /// The read concern to use for this find query.
     ///
     /// If none specified, the default set on the collection will be used.
-    #[builder(default)]
     pub read_concern: Option<ReadConcern>,
 
     /// Whether to return only the index keys in the documents.
-    #[builder(default)]
     pub return_key: Option<bool>,
 
     /// The criteria used to select a server for this find query.
     ///
     /// If none specified, the default set on the collection will be used.
-    #[builder(default)]
     pub selection_criteria: Option<SelectionCriteria>,
 
     /// Whether to return the record identifier for each document.
-    #[builder(default)]
     pub show_record_id: Option<bool>,
 
     /// The number of documents to skip before counting.
-    #[builder(default)]
-    pub skip: Option<i64>,
+    #[serde(serialize_with = "bson_util::serialize_u64_option_as_i64")]
+    pub skip: Option<u64>,
 
     /// The order of the documents for the purposes of the operation.
-    #[builder(default)]
     pub sort: Option<Document>,
 }
 
+/// Specifies the options to a
+/// [`Collection::create_index`](../struct.Collection.html#method.create_index) or [`Collection::
+/// create_indexes`](../struct.Collection.html#method.create_indexes) operation.
+///
+/// For more information, see [`createIndexes`](https://docs.mongodb.com/manual/reference/command/createIndexes/).
+#[serde_with::skip_serializing_none]
+#[derive(Clone, Debug, Default, TypedBuilder, Serialize)]
+#[builder(field_defaults(default, setter(into)))]
+#[serde(rename_all = "camelCase", deny_unknown_fields)]
+#[non_exhaustive]
+pub struct CreateIndexOptions {
+    /// Specify the commit quorum needed to mark an `index` as ready.
+    pub commit_quorum: Option<CommitQuorum>,
+
+    /// The maximum amount of time to allow the index to build.
+    ///
+    /// This option maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
+    /// across the wire as an integer number of milliseconds.
+    #[serde(
+        rename = "maxTimeMS",
+        default,
+        serialize_with = "bson_util::serialize_duration_option_as_int_millis",
+        deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis"
+    )]
+    pub max_time: Option<Duration>,
+
+    /// The write concern for the operation.
+    pub write_concern: Option<WriteConcern>,
+}
+
 /// Specifies the options to a [`Collection::drop`](../struct.Collection.html#method.drop)
 /// operation.
-#[derive(Debug, Default, TypedBuilder, Serialize)]
+#[serde_with::skip_serializing_none]
+#[derive(Clone, Debug, Default, Deserialize, TypedBuilder, Serialize)]
 #[serde(rename_all = "camelCase")]
+#[builder(field_defaults(default, setter(into)))]
 #[non_exhaustive]
 pub struct DropCollectionOptions {
     /// The write concern for the operation.
-    #[builder(default)]
     pub write_concern: Option<WriteConcern>,
 }
+
+/// Specifies the options to a
+/// [`Collection::drop_index`](../struct.Collection.html#method.drop_index) or
+/// [`Collection::drop_indexes`](../struct.Collection.html#method.drop_indexes) operation.
+#[serde_with::skip_serializing_none]
+#[derive(Clone, Debug, Default, Deserialize, TypedBuilder, Serialize)]
+#[serde(rename_all = "camelCase", deny_unknown_fields)]
+#[builder(field_defaults(default, setter(into)))]
+#[non_exhaustive]
+pub struct DropIndexOptions {
+    /// The maximum amount of time to allow the index to drop.
+    ///
+    /// This option maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
+    /// across the wire as an integer number of milliseconds.
+    #[serde(
+        rename = "maxTimeMS",
+        default,
+        serialize_with = "bson_util::serialize_duration_option_as_int_millis",
+        deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis"
+    )]
+    pub max_time: Option<Duration>,
+
+    /// The write concern for the operation.
+    pub write_concern: Option<WriteConcern>,
+}
+
+/// Specifies the options to a
+/// [`Collection::list_indexes`](../struct.Collection.html#method.list_indexes) operation.
+#[serde_with::skip_serializing_none]
+#[derive(Clone, Debug, Default, Deserialize, TypedBuilder, Serialize)]
+#[serde(rename_all = "camelCase", deny_unknown_fields)]
+#[builder(field_defaults(default, setter(into)))]
+#[non_exhaustive]
+pub struct ListIndexesOptions {
+    /// The maximum amount of time to search for the index.
+    ///
+    /// This option maps to the `maxTimeMS` MongoDB query option, so the duration will be sent
+    /// across the wire as an integer number of milliseconds.
+    #[serde(
+        rename = "maxTimeMS",
+        default,
+        serialize_with = "bson_util::serialize_duration_option_as_int_millis",
+        deserialize_with = "bson_util::deserialize_duration_option_from_u64_millis"
+    )]
+    pub max_time: Option<Duration>,
+
+    /// The number of indexes the server should return per cursor batch.
+    #[serde(default, serialize_with = "bson_util::serialize_u32_option_as_i32")]
+    pub batch_size: Option<u32>,
+}
+
+/// The minimum number of data-bearing voting replica set members (i.e. commit quorum), including
+/// the primary, that must report a successful index build before the primary marks the indexes as
+/// ready.
+///
+/// For more information, see the [documentation](https://docs.mongodb.com/manual/reference/command/createIndexes/#definition)
+#[derive(Clone, Debug, PartialEq)]
+#[non_exhaustive]
+pub enum CommitQuorum {
+    /// A specific number of voting replica set members. When set to 0, disables quorum voting.
+    Nodes(u32),
+
+    /// All data-bearing voting replica set members (default).
+    VotingMembers,
+
+    /// A simple majority of voting members.
+    Majority,
+
+    /// A replica set tag name.
+    Custom(String),
+}
+
+impl Serialize for CommitQuorum {
+    fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        match self {
+            CommitQuorum::Nodes(n) => serde_helpers::serialize_u32_as_i32(n, serializer),
+            CommitQuorum::VotingMembers => serializer.serialize_str("votingMembers"),
+            CommitQuorum::Majority => serializer.serialize_str("majority"),
+            CommitQuorum::Custom(s) => serializer.serialize_str(s),
+        }
+    }
+}
+
+impl<'de> Deserialize<'de> for CommitQuorum {
+    fn deserialize<D>(deserializer: D) -> std::result::Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        #[derive(Deserialize)]
+        #[serde(untagged)]
+        enum IntOrString {
+            Int(u32),
+            String(String),
+        }
+        match IntOrString::deserialize(deserializer)? {
+            IntOrString::String(s) => {
+                if s == "votingMembers" {
+                    Ok(CommitQuorum::VotingMembers)
+                } else if s == "majority" {
+                    Ok(CommitQuorum::Majority)
+                } else {
+                    Ok(CommitQuorum::Custom(s))
+                }
+            }
+            IntOrString::Int(i) => Ok(CommitQuorum::Nodes(i)),
+        }
+    }
+}
